﻿namespace Yo.Web
{
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.IO;
    using System.Net.Mail;
    using System.ServiceModel.DomainServices.Hosting;
    using System.ServiceModel.DomainServices.Server;
    using System.Text;
    using System.Web.Profile;
    using System.Web.Security;
    using Yo.Web.Resources;

    // TODO: Switch to a secure endpoint when deploying the application.
    //       The user's name and password should only be passed using https.
    //       To do this, set the RequiresSecureEndpoint property on EnableClientAccessAttribute to true.
    //   
    //       [EnableClientAccess(RequiresSecureEndpoint = true)]
    //
    //       More information on using https with a Domain Service can be found on MSDN.

    /// <summary>
    /// Domain Service responsible for registering users.
    /// </summary>
         [EnableClientAccess]
         public class UserRegistrationService : DomainService
    {
        public const string DefaultRole = "Registered Users";
        public bool Authenticate(string userName, string password)
        {
            return Membership.ValidateUser(userName, password);
        }

        //// NOTE: This is a sample code to get your application started. In the production code you would 
        //// want to provide a mitigation against a denial of service attack by providing CAPTCHA 
        //// control functionality or verifying user's email address.
       [Invoke ]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        public void AddUser(RegistrationData user, string ImgAbsolutePath)
        {
            // Run this BEFORE creating the user to make sure roles are enabled and the default role
            // will be available
            //
            // If there are a problem with the role manager it is better to fail now than to have it
            // happening after the user is created
            if (!Roles.RoleExists(UserRegistrationService.DefaultRole))
            {
                Roles.CreateRole(UserRegistrationService.DefaultRole);
            }

            // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
            // CreateUser will fail if you do not have SQL Server Express installed.
            MembershipCreateStatus createStatus;
            Membership.CreateUser(user.UserName, user.Password, user.Email, user.Question, user.Answer, true, null, out createStatus);
            // Membership.UpdateUser(user)
            if (createStatus != MembershipCreateStatus.Success)
            {
                throw new DomainException(ErrorCodeToString(createStatus));
            }

            // Assign it to the default role
            // This *can* fail but only if role management is disabled
            Roles.AddUserToRole(user.UserName, UserRegistrationService.DefaultRole);

            //// Set its friendly name (profile setting)
            //// This *can* fail but only if Web.config is configured incorrectly 
            
            if (user.ImageSource == null)
            {
                user.ImageURL = SaveImageFromFileURL(user.UserName, ImgAbsolutePath);
            }
            else
            {

                user.ImageURL = SaveImageFile(user.UserName, user.ImageSource, ImgAbsolutePath);
            }
            ProfileBase profile = ProfileBase.Create(user.UserName, true);
            profile.SetPropertyValue("DateCreated", DateTime.Now);

            // profile.SetPropertyValue("User_Name", user.UserName);
            profile.SetPropertyValue("LastUpdate", DateTime.Now);
            profile.SetPropertyValue("First_Name", user.First_Name);
            profile.SetPropertyValue("Last_Name", user.Last_Name);
            profile.SetPropertyValue("DOB", user.DOB);
            profile.SetPropertyValue("Email", user.Email);
            profile.SetPropertyValue("ImageUrl", user.ImageURL);
            profile.SetPropertyValue("Gender", user.Gender);
            profile.SetPropertyValue("Country", user.Country);
            profile.SetPropertyValue("State", user.State);
            profile.SetPropertyValue("User_Active", true);
            profile.SetPropertyValue("DateCreated", DateTime.Now);
            profile.Save();
        }
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        [Invoke]
        public void ImplementUserProfile(RegistrationData user, string ImgAbsolutePath)
        {

            // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
            // CreateUser will fail if you do not have SQL Server Express installed.
            

            //MembershipUser membershipuser = new MembershipUser();
            //membershipuser.ChangePasswordQuestionAndAnswer(
            //Membership.UpdateUser(user.UserName, user.Password, user.Email, user.Question, user.Answer, true, null, out createStatus);
            //// Membership.UpdateUser(user)
            //if (createStatus != MembershipCreateStatus.Success)
            //{
            //    throw new DomainException(ErrorCodeToString(createStatus));
            //}

            //// Set its friendly name (profile setting)
            //// This *can* fail but only if Web.config is configured incorrectly 


            if (user.ImageSource == null)
            {
                user.ImageURL = SaveImageFromFileURL(user.UserName,ImgAbsolutePath);
            }
            else
            {
                user.ImageURL = SaveImageFile(user.UserName, user.ImageSource, ImgAbsolutePath);
            }
            ProfileBase profile = ProfileBase.Create(user.UserName, true);
            profile.SetPropertyValue("DateCreated", DateTime.Now);
            // profile.SetPropertyValue("User_Name", user.UserName);
            profile.SetPropertyValue("LastUpdate", DateTime.Now);
            profile.SetPropertyValue("First_Name", user.First_Name);
            profile.SetPropertyValue("Last_Name", user.Last_Name);
            profile.SetPropertyValue("DOB", user.DOB);
            profile.SetPropertyValue("Email", user.Email);
            profile.SetPropertyValue("ImageUrl", user.ImageURL);
            profile.SetPropertyValue("Gender", user.Gender);
            profile.SetPropertyValue("Country", user.Country);
            profile.SetPropertyValue("State", user.State);
            profile.SetPropertyValue("User_Active", true);

            profile.Save();

        }


        [Update]
        public void UpdateUser(RegistrationData user)
        {

            // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
            // CreateUser will fail if you do not have SQL Server Express installed.
            //MembershipCreateStatus createStatus;

            //MembershipUser membershipuser = new MembershipUser();
            //membershipuser.ChangePasswordQuestionAndAnswer(
            //Membership.UpdateUser(user.UserName, user.Password, user.Email, user.Question, user.Answer, true, null, out createStatus);
            //// Membership.UpdateUser(user)
            //if (createStatus != MembershipCreateStatus.Success)
            //{
            //    throw new DomainException(ErrorCodeToString(createStatus));
            //}

            //// Set its friendly name (profile setting)
            //// This *can* fail but only if Web.config is configured incorrectly 
            string ImageAbsolutePath = System.Configuration.ConfigurationManager.AppSettings["ImageGalleryAbsPath"];
            if (user.ImageSource == null)
            {
                user.ImageURL = SaveImageFromFileURL(user.UserName,ImageAbsolutePath);
            }
            else
            {
                user.ImageURL = SaveImageFile(user.UserName, user.ImageSource, ImageAbsolutePath);
            }

            ProfileBase profile = ProfileBase.Create(user.UserName, true);
            profile.SetPropertyValue("DateCreated", DateTime.Now);
            // profile.SetPropertyValue("User_Name", user.UserName);
            profile.SetPropertyValue("LastUpdate", DateTime.Now);
            profile.SetPropertyValue("First_Name", user.First_Name);
            profile.SetPropertyValue("Last_Name", user.Last_Name);
            profile.SetPropertyValue("DOB", user.DOB);
            profile.SetPropertyValue("Email", user.Email);
            profile.SetPropertyValue("ImageUrl", user.ImageURL);
            profile.SetPropertyValue("Gender", user.Gender);
            profile.SetPropertyValue("Country", user.Country);
            profile.SetPropertyValue("State", user.State);
            profile.SetPropertyValue("User_Active", true);

            profile.Save();

        }

        [Invoke]
        public void ChangePassword(string username, string oldpassword, string newpassword)
        {

            // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
            // CreateUser will fail if you do not have SQL Server Express installed.

            MembershipUser user = Membership.GetUser(username);
            bool isattemptsuccess = user.ChangePassword(oldpassword, newpassword);

            if (!isattemptsuccess)
            {
                throw new DomainException(ErrorResources.MembershipPasswordChangeError);
            }



        }


        [Invoke]
        public void ResetPassword(string username, string question, string answer)
        {

            // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
            // CreateUser will fail if you do not have SQL Server Express installed.

            MembershipUser user = Membership.GetUser(username);
            if (user.PasswordQuestion != question)
            {

                throw new DomainException(ErrorResources.MembershipQuestionMismatch);
            }
            string newpassword = user.ResetPassword(answer);

            //  ToDo : send email
            //use.Email
            sentEmail(newpassword, user.Email);

        }
        [Invoke]
        public string GetSecurityQuestion(string username)
        {

            // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
            // CreateUser will fail if you do not have SQL Server Express installed.

            MembershipUser user = Membership.GetUser(username);
            return user.PasswordQuestion;



        }

        [Invoke]
        public void ChangeSecurityQuestionPassword(string username, string password, string question, string answer)
        {

            // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
            // CreateUser will fail if you do not have SQL Server Express installed.

            MembershipUser user = Membership.GetUser(username);
            bool isattemptsuccess = user.ChangePasswordQuestionAndAnswer(password, question, answer);
            if (!isattemptsuccess)
            {
                //throw new DomainException(ErrorCodeToString(MembershipCreateStatus.InvalidPassword));
                throw new DomainException(ErrorResources.MembershipPasswordChangeError);

            }



        }

        public string ImageAbsolutePath { get; set; }
        public string ImageRelURL { get; set; }

        [Invoke]
        public string[] GetImagesOfPerson(string currentperson, string ImageOrgURL, string ImageOrgURLURL)
        {
            string[] arrImages = null;
            ImageAbsolutePath = ImageOrgURL;
            ImageRelURL = ImageOrgURLURL;
            //todo
             //ImageOrgURL = System.Configuration.ConfigurationManager.AppSettings["ImageGalleryAbsPath"];
             //ImageOrgURLURL = System.Configuration.ConfigurationManager.AppSettings["ImageGalleryPath"];
            string ImageURL = ImageOrgURL + @"\" + currentperson + @"\";
            string[] imgPaths = null;
            if (System.IO.Directory.Exists(ImageURL))
            {
                arrImages = System.IO.Directory.GetFiles(ImageURL);

                imgPaths = new string[arrImages.Length];

                for (int i = 0; i < arrImages.Length; i++)
                {
                    imgPaths[i] = arrImages[i].Replace(ImageOrgURL, ImageOrgURLURL).Replace("\\", "/");

                }

            }


            return imgPaths;

        }

        [Invoke]
        public string SaveImageToGallery(string personname, string fileName, Byte[] stream, string ImageOrgURL)
        {
            //string filePath =   System.Configuration.ConfigurationManager.AppSettings["ImageGalleryAbsPath"];
            string filePath = ImageOrgURL + @"\" + personname + @"\";
            //create a folder if not exist
            if (!System.IO.Directory.Exists(filePath))
            {
                System.IO.Directory.CreateDirectory(filePath);
            }
            filePath = filePath + fileName;


            FileStream fileSave = new FileStream(filePath, FileMode.OpenOrCreate,FileAccess.ReadWrite);


            fileSave.Write(stream, 0, (int)stream.Length);

            fileSave.Close();


            return filePath;
        }


        [Invoke]
        public string DeleteImageToGallery(string filename, string ImageOrgURL)
        {
            //string ImageOrgURL = System.Configuration.ConfigurationManager.AppSettings["ImageGalleryAbsPath"];

            string imagePath = ImageOrgURL + filename;
            imagePath = imagePath.Replace(@"\", "/");
            //create a folder if not exist
            if (System.IO.File.Exists(imagePath))
            {
                System.IO.File.Delete(imagePath);
            }



            return filename;
        }

        public string SaveImageFile(string fileName, Byte[] stream , string ImgAbsolutePath)
        {
            string filePath = ImgAbsolutePath;//System.Configuration.ConfigurationManager.AppSettings["ImageGalleryAbsPath"];
             filePath = filePath + @"\" + fileName + @"\";
            //create a folder if not exist

          
            if (!System.IO.Directory.Exists(filePath))
            {
                System.IO.Directory.CreateDirectory(filePath);
            }
            filePath = filePath + fileName + ".jpg";


            FileStream fileSave = new FileStream(filePath, FileMode.OpenOrCreate,FileAccess.ReadWrite);


            fileSave.Write(stream, 0, (int)stream.Length);

            fileSave.Close();


            return filePath;
        }


        private byte[] GetBytesFromImage(string fileName)
        {


            FileStream fileFrom = new FileStream(fileName, FileMode.OpenOrCreate, FileAccess.ReadWrite);
            int bytelenght = (int)fileFrom.Length;
            byte[] bytes = new byte[bytelenght];

            fileFrom.Read(bytes, 0, bytelenght);




            fileFrom.Close();


            return bytes;
        }


        public string SaveImageFromFileURL(string fileName,string ImageAbsolutePath)
        {
            string filePath = ImageAbsolutePath  ; //System.Configuration.ConfigurationManager.AppSettings["ImageGalleryAbsPath"];
            string fileFromPath = ImageAbsolutePath + "dummy.jpg";

            
            //create a folder if not exist
            if (!System.IO.Directory.Exists(filePath))
            {
                System.IO.Directory.CreateDirectory(filePath);
            }
            filePath = filePath + fileName + ".jpg";



            FileStream fileSave = new FileStream(filePath, FileMode.OpenOrCreate,FileAccess.ReadWrite);

            FileStream fileFrom = new FileStream(fileFromPath, FileMode.Open,FileAccess.Read);

            int bytelenght = (int)fileFrom.Length;
            byte[] bytes = new byte[bytelenght];

            fileFrom.Read(bytes, 0, bytelenght);
            fileSave.Write(bytes, 0, bytelenght);

            fileSave.Close();
            fileFrom.Close();

            return filePath;
        }

        private void sentEmail(string _password, string ToEmail)
        {

            

            String FromE = ConfigurationManager.AppSettings.Get("FromE");


            String Mail_Password = ConfigurationManager.AppSettings.Get("MailEmailPassword");

            String Mail_Username = ConfigurationManager.AppSettings.Get("MailEmailID");

            int Mailport = Convert.ToInt32(ConfigurationManager.AppSettings.Get("MailEmailport"));

            String Server_Host = Convert.ToString(ConfigurationManager.AppSettings.Get("MailEmailHost"));



            try
            {

                MailMessage mm = new MailMessage();

                SmtpClient smtp = new SmtpClient();

                mm.From = new MailAddress(FromE);

                //add the Emails to a To Collection

                mm.To.Add(new MailAddress(ToEmail));

                mm.Subject = "SharePointNest New Password Email";

                mm.Body = BuildBody(ToEmail, _password);

                mm.IsBodyHtml = true;

                smtp.Host = Server_Host;

                System.Net.NetworkCredential NetworkCred = new System.Net.NetworkCredential();

                NetworkCred.UserName = Mail_Username;

                NetworkCred.Password = Mail_Password;

                smtp.Credentials = NetworkCred;

                smtp.Port = Mailport;

                smtp.Send(mm);


            }
            catch (System.Net.Mail.SmtpException)
            {

                throw;


            }
        }

        private String BuildBody(String ClientEmail, string ClientPassword)
        {
            StringBuilder Str = new StringBuilder();

            Str.Append("This message was created by SharePointNest Site Administration." + "<br><br>");

            Str.Append("*** PLEASE DO NOT REPLY THIS MESSAGE ***" + "<br><br>");

            Str.Append("Please login using the following information:" + "<br><br>");

            Str.Append("Login Email   :" + ClientEmail + "<br><br>");

            Str.Append("Password:" + ClientPassword + "<br><br>");
            Str.Append("You can use these login credentials to login to SharePointNest <br><br>");
            Str.Append("Thank you for using and supporting  to SharePointNest <br><br>");

           

            
            Str.Append("Kind Regards" + "<br><br><br>");

            Str.Append("SharePointNest" + "<br>");

            return Str.ToString();

        }

        [Delete]
        public void DeleteUser(RegistrationData user)
        {

            // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
           

            //MembershipUser membershipuser = new MembershipUser();
            //membershipuser.ChangePasswordQuestionAndAnswer(
            //Membership.UpdateUser(user.UserName, user.Password, user.Email, user.Question, user.Answer, true, null, out createStatus);
            //// Membership.UpdateUser(user)
            //if (createStatus != MembershipCreateStatus.Success)
            //{
            //    throw new DomainException(ErrorCodeToString(createStatus));
            //}

            //// Set its friendly name (profile setting)
            //// This *can* fail but only if Web.config is configured incorrectly 
            ProfileBase profile = ProfileBase.Create(user.UserName, true);
            string ImageAbsolutePath = System.Configuration.ConfigurationManager.AppSettings["ImageGalleryAbsPath"];
            if (user.ImageSource == null)
            {
                user.ImageURL = SaveImageFromFileURL(user.UserName, ImageAbsolutePath);
            }
            else
            {
                user.ImageURL = SaveImageFile(user.UserName, user.ImageSource,ImageAbsolutePath);
            }
            profile.SetPropertyValue("DateCreated", DateTime.Now);
            // profile.SetPropertyValue("User_Name", user.UserName);
            profile.SetPropertyValue("LastUpdate", DateTime.Now);
            profile.SetPropertyValue("First_Name", user.First_Name);
            profile.SetPropertyValue("Last_Name", user.Last_Name);
            profile.SetPropertyValue("DOB", user.DOB);
            profile.SetPropertyValue("Email", user.Email);
            profile.SetPropertyValue("ImageUrl", user.ImageURL);
            profile.SetPropertyValue("Gender", user.Gender);
            profile.SetPropertyValue("Country", user.Country);
            profile.SetPropertyValue("State", user.State);
            profile.SetPropertyValue("User_Active", true);

            profile.Save();

        }




        // This is never used but without it RIA Services will complain RegistrationData
        // is not exposed as an entity
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        public IEnumerable<RegistrationData> GetUsers()
        {
            throw new NotSupportedException();
        }


        // This is never used but without it RIA Services will complain RegistrationData
        // is not exposed as an entity
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        public RegistrationData GetUser(string username)
        {  // NOTE: ASP.NET by default uses SQL Server Express to create the user database. 
            // CreateUser will fail if you do not have SQL Server Express installed.


            RegistrationData data = new RegistrationData();
            MembershipUser user = Membership.GetUser(username);
            if (user != null)
            {
                data.Email = user.Email;
                data.Question = user.PasswordQuestion;
                data.UserName = user.UserName;
                data.Password = user.GetPassword();
                ProfileBase profile = ProfileBase.Create(username, true);





                data.First_Name = profile.GetPropertyValue("First_Name").ToString();
                data.Last_Name = profile.GetPropertyValue("Last_Name").ToString();
                data.DOB = profile.GetPropertyValue("DOB").ToString() == string.Empty ? DateTime.MinValue : Convert.ToDateTime(profile.GetPropertyValue("DOB").ToString());
                data.Email = profile.GetPropertyValue("Email").ToString();
                data.ImageURL = profile.GetPropertyValue("ImageUrl").ToString();
                //data.ImageSource = GetBytesFromImage(data.ImageURL);
                data.Gender = profile.GetPropertyValue("Gender").ToString();
                data.Country = profile.GetPropertyValue("Country").ToString();
                data.State = profile.GetPropertyValue("State").ToString();

            }
            return data;
        }


        private static string ErrorCodeToString(MembershipCreateStatus createStatus)
        {
            // See http://msdn.microsoft.com/en-us/library/system.web.security.membershipcreatestatus.aspx for
            // a full list of status codes and add appropriate error handling.
            switch (createStatus)
            {
                case MembershipCreateStatus.DuplicateUserName:
                    return ClientCultureBasedResources.GetResource(() => ErrorResources.MembershipCreateStatusDuplicateUserName);

                case MembershipCreateStatus.DuplicateEmail:
                    return ClientCultureBasedResources.GetResource(() => ErrorResources.MembershipCreateStatusDuplicateEmail);

                case MembershipCreateStatus.ProviderError:
                    return ClientCultureBasedResources.GetResource(() => ErrorResources.MembershipCreateStatusProviderError);

                case MembershipCreateStatus.UserRejected:
                    return ClientCultureBasedResources.GetResource(() => ErrorResources.MembershipCreateStatusUserRejected);

                case MembershipCreateStatus.InvalidPassword:
                    return ClientCultureBasedResources.GetResource(() => ErrorResources.MembershipPasswordChangeError);

                case MembershipCreateStatus.InvalidEmail:
                case MembershipCreateStatus.InvalidAnswer:
                case MembershipCreateStatus.InvalidQuestion:
                case MembershipCreateStatus.InvalidUserName:
                    // All this errors should have been handled by the UI validation so theoretically
                    // we should never get to this point
                    return "Validation Error: " + createStatus.ToString();

                default:
                    return "Could not register the user, please verify the provided information and try again.";
            }
        }
             private static CreateUserStatus ConvertStatus(MembershipCreateStatus createStatus)
        {
            switch (createStatus)
            {
                case MembershipCreateStatus.Success: return CreateUserStatus.Success;
                case MembershipCreateStatus.InvalidUserName: return CreateUserStatus.InvalidUserName;
                case MembershipCreateStatus.InvalidPassword: return CreateUserStatus.InvalidPassword;
                case MembershipCreateStatus.InvalidQuestion: return CreateUserStatus.InvalidQuestion;
                case MembershipCreateStatus.InvalidAnswer: return CreateUserStatus.InvalidAnswer;
                case MembershipCreateStatus.InvalidEmail: return CreateUserStatus.InvalidEmail;
                case MembershipCreateStatus.DuplicateUserName: return CreateUserStatus.DuplicateUserName;
                case MembershipCreateStatus.DuplicateEmail: return CreateUserStatus.DuplicateEmail;
                default: return CreateUserStatus.Failure;
            }
        }

             [Invoke(HasSideEffects = true)]
             public CreateUserStatus CreateUserTest(){
 

                 return CreateUserStatus.Success;
             }
        
        }
        /// <summary>
        /// An enumeration of the values that can be returned from <see cref="DomainService1.CreateUser"/>
        /// </summary>
        
    
        public enum CreateUserStatus
        {
            Success = 0,
            InvalidUserName = 1,
            InvalidPassword = 2,
            InvalidQuestion = 3,
            InvalidAnswer = 4,
            InvalidEmail = 5,
            DuplicateUserName = 6,
            DuplicateEmail = 7,
            Failure = 8,
        }
    
}